from __future__ import annotations

from eightmem.core.paths import memory_dir
from eightmem.core.templates import copy_default_templates


def test_init_creates_templates(fake_home):
    mem = memory_dir()
    mem.mkdir(parents=True, exist_ok=True)
    created = copy_default_templates(mem)

    assert created
    assert (mem / "IDENTITY.md").exists()
    assert (mem / "DECISIONS.md").exists()
